@@ -1,4 +1,3 @@
-/* $Id: xfburn-audio-composition.c 5905 2008-11-02 05:32:28Z squisher $ */
 /*
  * Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -33,11 +32,9 @@
 
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
-#ifdef HAVE_THUNAR_VFS
-#include <thunar-vfs/thunar-vfs.h>
-#endif
+#include <gio/gio.h>
 
 #include <exo/exo.h>
 
@@ -46,6 +43,7 @@
 #include "xfburn-audio-composition.h"
 #include "xfburn-global.h"
 #include "xfburn-error.h"
+#include "xfburn-utils.h"
 
 #include "xfburn-adding-progress.h"
 #include "xfburn-composition.h"
@@ -54,6 +52,7 @@
 #include "xfburn-utils.h"
 #include "xfburn-burn-audio-cd-composition-dialog.h"
 #include "xfburn-transcoder.h"
+#include "xfburn-settings.h"
 
 #define XFBURN_AUDIO_COMPOSITION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), XFBURN_TYPE_AUDIO_COMPOSITION, XfburnAudioCompositionPrivate))
 
@@ -77,7 +76,7 @@ enum
 {
   AUDIO_COMPOSITION_DISPLAY_COLUMN_POS,
   AUDIO_COMPOSITION_DISPLAY_COLUMN_LENGTH,
-#if 0 // Needs libburn cd-text suppport
+#if 0 /* CDTEXT */
   AUDIO_COMPOSITION_DISPLAY_COLUMN_ARTIST,
   AUDIO_COMPOSITION_DISPLAY_COLUMN_TITLE,
 #endif
@@ -93,8 +92,7 @@ typedef enum
 
 /* thread parameters */
 typedef struct {
-  char **filenames;
-  int filec;
+  GSList * filelist;
   XfburnAudioComposition *dc;
 } ThreadAddFilesCLIParams;
 
@@ -119,6 +117,7 @@ static void composition_interface_init (XfburnCompositionInterface *composition,
 static void xfburn_audio_composition_init (XfburnAudioComposition *dc);
 static void xfburn_audio_composition_finalize (GObject * object);
 
+/* internals */
 static void show_custom_controls (XfburnComposition *composition);
 static void hide_custom_controls (XfburnComposition *composition);
 static void load_from_file (XfburnComposition *composition, const gchar *file);
@@ -127,9 +126,12 @@ static void save_to_file (XfburnComposition *composition);
 //static gint song_tree_sortfunc (GtkTreeModel * model, GtkTreeIter * a, GtkTreeIter * b, gpointer user_data);
 
 static void action_clear (GtkAction *, XfburnAudioComposition *);
+static void action_info (GtkAction *, XfburnAudioComposition *);
 static void action_remove_selection (GtkAction *, XfburnAudioComposition *);
+#if 0 /* CDTEXT */
 static void action_rename_selection_artist (GtkAction *, XfburnAudioComposition *);
 static void action_rename_selection_title (GtkAction *, XfburnAudioComposition *);
+#endif /* CDTEXT */
 static void action_add_selected_files (GtkAction *, XfburnAudioComposition *);
 
 static void tracks_changed (XfburnAudioComposition *ac);
@@ -137,7 +139,7 @@ static gboolean cb_treeview_button_pressed (GtkTreeView * treeview, GdkEventButt
 static void cb_selection_changed (GtkTreeSelection *selection, XfburnAudioComposition * dc);
 static GSList * generate_audio_src (XfburnAudioComposition * ac);
 static void cb_begin_burn (XfburnDiscUsage * du, XfburnAudioComposition * dc);
-#if 0 // Needs libburn cd-text suppport
+#if 0 /* CDTEXT */
 static void cb_cell_artist_edited (GtkCellRenderer * renderer, gchar * path, gchar * newtext, XfburnAudioComposition * dc);
 static void cb_cell_title_edited (GtkCellRenderer * renderer, gchar * path, gchar * newtext, XfburnAudioComposition * dc);
 #endif
@@ -145,7 +147,7 @@ static void cb_cell_title_edited (GtkCellRenderer * renderer, gchar * path, gcha
 static void cb_content_drag_data_rcv (GtkWidget * widget, GdkDragContext * dc, guint x, guint y, GtkSelectionData * sd,
                                       guint info, guint t, XfburnAudioComposition * composition);
 static void cb_content_drag_data_get (GtkWidget * widget, GdkDragContext * dc, GtkSelectionData * data, guint info,
-                                      guint time, XfburnAudioComposition * content);
+                                      guint timestamp, XfburnAudioComposition * content);
 static void cb_adding_done (XfburnAddingProgress *progress, XfburnAudioComposition *dc);
 
 /* thread entry points */
@@ -159,6 +161,7 @@ static gboolean thread_add_file_to_list_with_name (const gchar *name, XfburnAudi
                                                    GtkTreeIter * insertion, GtkTreeViewDropPosition position);
 static gboolean thread_add_file_to_list (XfburnAudioComposition * dc, GtkTreeModel * model, const gchar * path, 
                                          GtkTreeIter * iter, GtkTreeIter * insertion, GtkTreeViewDropPosition position);
+static gboolean show_add_home_question_dialog (void);
                                   
 typedef struct
 {
@@ -187,14 +190,17 @@ typedef struct
   GtkWidget *content;
   GtkWidget *disc_usage;
   GtkWidget *progress;
+  GtkTreeStore *model;
 
   XfburnTranscoder *trans;
 } XfburnAudioCompositionPrivate;
 
 /* globals */
+#define MAX_NAME_LENGTH 80
 static GtkHPanedClass *parent_class = NULL;
 static guint instances = 0;
 static gchar *did_warn = "Did warn about this already";
+static gchar trans_name[MAX_NAME_LENGTH] = {""};
 
 static const GtkActionEntry action_entries[] = {
   {"add-file", GTK_STOCK_ADD, N_("Add"), NULL, N_("Add the selected file(s) to the composition"),
@@ -203,11 +209,15 @@ static const GtkActionEntry action_entries[] = {
    G_CALLBACK (action_remove_selection),},
   {"clear", GTK_STOCK_CLEAR, N_("Clear"), NULL, N_("Clear the content of the composition"),
    G_CALLBACK (action_clear),},
+  {"transcoder-info", GTK_STOCK_INFO, trans_name, NULL, N_("What files can get burned to an audio CD?"),
+   G_CALLBACK (action_info),},
   //{"import-session", "xfburn-import-session", N_("Import"), NULL, N_("Import existing session"),},
+#if 0 /* CDTEXT */
   {"rename-artist", GTK_STOCK_EDIT, N_("Rename Artist"), NULL, N_("Rename the artist of the selected file"),
    G_CALLBACK (action_rename_selection_artist),},
   {"rename-title", GTK_STOCK_EDIT, N_("Rename Title"), NULL, N_("Rename the title of the selected file"),
    G_CALLBACK (action_rename_selection_title),},
+#endif /* CDTEXT */
 };
 
 static const gchar *toolbar_actions[] = {
@@ -215,6 +225,7 @@ static const gchar *toolbar_actions[] = {
   "remove-file",
   "clear",
   "import-session",
+  "transcoder-info",
 };
 
 static GdkPixbuf *icon_directory = NULL, *icon_file = NULL;
@@ -222,10 +233,10 @@ static GdkPixbuf *icon_directory = NULL, *icon_file = NULL;
 /********************************/
 /* XfburnAudioComposition class */
 /********************************/
-GtkType
+GType
 xfburn_audio_composition_get_type (void)
 {
-  static GtkType audio_composition_type = 0;
+  static GType audio_composition_type = 0;
 
   if (!audio_composition_type) {
     static const GTypeInfo audio_composition_info = {
@@ -288,7 +299,7 @@ xfburn_audio_composition_init (XfburnAudioComposition * composition)
   //GtkWidget *hbox, *label;
   GtkWidget *scrolled_window;
   GtkTreeStore *model;
-#if 0 // Needs libburn cd-text suppport
+#if 0 /* CDTEXT */
   GtkTreeViewColumn *column_artist, *column_title;
   GtkCellRenderer *cell_artist, *cell_title;
 #endif
@@ -297,16 +308,24 @@ xfburn_audio_composition_init (XfburnAudioComposition * composition)
   GdkScreen *screen;
   GtkIconTheme *icon_theme;
   
+#if 0 /* CDTEXT */
   const gchar ui_string[] = "<ui> <popup name=\"popup-menu\">"
     "<menuitem action=\"rename-artist\"/>" "<menuitem action=\"rename-title\"/>" "<menuitem action=\"remove-file\"/>" "</popup></ui>";
+#else
+  const gchar ui_string[] = "<ui> <popup name=\"popup-menu\"> <menuitem action=\"remove-file\"/>" "</popup></ui>";
+#endif /* CDTEXT */
 
   GtkTargetEntry gte_src[] =  { { "XFBURN_TREE_PATHS", GTK_TARGET_SAME_WIDGET, AUDIO_COMPOSITION_DND_TARGET_INSIDE } };
   GtkTargetEntry gte_dest[] = { { "XFBURN_TREE_PATHS", GTK_TARGET_SAME_WIDGET, AUDIO_COMPOSITION_DND_TARGET_INSIDE },
-                                { "text/plain", 0, AUDIO_COMPOSITION_DND_TARGET_TEXT_PLAIN },
+                                { "text/plain;charset=utf-8", 0, AUDIO_COMPOSITION_DND_TARGET_TEXT_PLAIN },
                                 { "text/uri-list", 0, AUDIO_COMPOSITION_DND_TARGET_TEXT_URI_LIST },
                               };
 
   priv->full_paths_to_add = NULL;
+  priv->trans = xfburn_transcoder_get_global ();
+
+  if (trans_name[0] == '\0')
+    strncpy (trans_name, xfburn_transcoder_get_name (priv->trans), MAX_NAME_LENGTH);
 
   instances++;
   
@@ -345,6 +364,8 @@ xfburn_audio_composition_init (XfburnAudioComposition * composition)
   exo_toolbars_model_add_separator (model_toolbar, toolbar_position, -1);
   exo_toolbars_model_add_item (model_toolbar, toolbar_position, -1, "remove-file", EXO_TOOLBARS_ITEM_TYPE);
   exo_toolbars_model_add_item (model_toolbar, toolbar_position, -1, "clear", EXO_TOOLBARS_ITEM_TYPE);
+  exo_toolbars_model_add_separator (model_toolbar, toolbar_position, -1);
+  exo_toolbars_model_add_item (model_toolbar, toolbar_position, -1, "transcoder-info", EXO_TOOLBARS_ITEM_TYPE);
   //exo_toolbars_model_add_separator (model_toolbar, toolbar_position, -1);
   //exo_toolbars_model_add_item (model_toolbar, toolbar_position, -1, "import-session", EXO_TOOLBARS_ITEM_TYPE);
 
@@ -362,8 +383,9 @@ xfburn_audio_composition_init (XfburnAudioComposition * composition)
 
   priv->content = exo_tree_view_new ();
   model = gtk_tree_store_new (AUDIO_COMPOSITION_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_STRING,
-                              G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_POINTER);
-							  
+                              G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT, XFBURN_TYPE_AUDIO_TRACK);
+  priv->model = model;
+
   /*
   gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model), AUDIO_COMPOSITION_COLUMN_POS,
                                    song_tree_sortfunc, NULL, NULL);
@@ -383,7 +405,7 @@ xfburn_audio_composition_init (XfburnAudioComposition * composition)
                                                gtk_cell_renderer_text_new (), "text", AUDIO_COMPOSITION_COLUMN_HUMANLENGTH, NULL);
   /*gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->content), -1, _("Artist"),
                                                gtk_cell_renderer_text_new (), "text", AUDIO_COMPOSITION_COLUMN_ARTIST, NULL);*/
-#if 0 // Needs libburn cd-text suppport
+#if 0 /* CDTEXT */
   column_artist = gtk_tree_view_column_new ();
   gtk_tree_view_column_set_title (column_artist, _("Artist"));
 
@@ -418,7 +440,7 @@ xfburn_audio_composition_init (XfburnAudioComposition * composition)
   /* Length */
   gtk_tree_view_column_set_resizable (gtk_tree_view_get_column (GTK_TREE_VIEW (priv->content), AUDIO_COMPOSITION_DISPLAY_COLUMN_LENGTH), FALSE);
   gtk_tree_view_column_set_min_width (gtk_tree_view_get_column (GTK_TREE_VIEW (priv->content), AUDIO_COMPOSITION_DISPLAY_COLUMN_LENGTH), 60);
-#if 0 // Needs libburn cd-text suppport
+#if 0 /* CDTEXT */
   /* Artist */
   gtk_tree_view_column_set_resizable (gtk_tree_view_get_column (GTK_TREE_VIEW (priv->content), AUDIO_COMPOSITION_DISPLAY_COLUMN_ARTIST), TRUE);
   gtk_tree_view_column_set_min_width (gtk_tree_view_get_column (GTK_TREE_VIEW (priv->content), AUDIO_COMPOSITION_DISPLAY_COLUMN_ARTIST), 80);
@@ -459,8 +481,6 @@ xfburn_audio_composition_init (XfburnAudioComposition * composition)
   action = gtk_action_group_get_action (priv->action_group, "remove-file");
   gtk_action_set_sensitive (GTK_ACTION (action), FALSE);
 
-  priv->trans = xfburn_transcoder_get_global ();
-
   priv->warned_about = g_hash_table_new (g_direct_hash, g_direct_equal);
 }
 
@@ -484,10 +504,13 @@ xfburn_audio_composition_finalize (GObject * object)
     }
   }
 
-  /* FIXME: free XfburnAudioTracks */
-
   g_object_unref (priv->trans);
   priv->trans = NULL;
+
+  /* while the content treeview is part of the GUI and is automatically destroyed
+     when this tab is closed, it does not take ownership of the model, which then
+     must be unref'ed manually */
+  g_object_unref (priv->model);
   
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -555,7 +578,7 @@ cb_begin_burn (XfburnDiscUsage * du, XfburnAudioComposition * dc)
     dialog = xfburn_burn_audio_cd_composition_dialog_new (src);
     break;
   case DVD_DISC:
-    xfce_err (_("Cannot burn audio onto a DVD"));
+    xfce_dialog_show_error (NULL, NULL, _("Cannot burn audio onto a DVD."));
     return;
     break;
   }
@@ -617,8 +640,10 @@ cb_treeview_button_pressed (GtkTreeView * treeview, GdkEventButton * event, Xfbu
     GtkTreePath *path;
     GtkWidget *menu_popup;
     GtkWidget *menuitem_remove;
+#if 0 /* CDTEXT */
     GtkWidget *menuitem_rename_artist;
     GtkWidget *menuitem_rename_title;
+#endif /* CDTEXT */
 
     selection = gtk_tree_view_get_selection (treeview);
 
@@ -630,13 +655,16 @@ cb_treeview_button_pressed (GtkTreeView * treeview, GdkEventButton * event, Xfbu
 
     menu_popup = gtk_ui_manager_get_widget (priv->ui_manager, "/popup-menu");
     menuitem_remove = gtk_ui_manager_get_widget (priv->ui_manager, "/popup-menu/remove-file");
+#if 0 /* CDTEXT */
     menuitem_rename_artist = gtk_ui_manager_get_widget (priv->ui_manager, "/popup-menu/rename-artist");
     menuitem_rename_title = gtk_ui_manager_get_widget (priv->ui_manager, "/popup-menu/rename-title");
+#endif /* CDTEXT */
 
     if (gtk_tree_selection_count_selected_rows (selection) >= 1)
       gtk_widget_set_sensitive (menuitem_remove, TRUE);
     else
       gtk_widget_set_sensitive (menuitem_remove, FALSE);
+#if 0 /* CDTEXT */
     if (gtk_tree_selection_count_selected_rows (selection) == 1) {
       gtk_widget_set_sensitive (menuitem_rename_artist, TRUE);
       gtk_widget_set_sensitive (menuitem_rename_title, TRUE);
@@ -644,6 +672,7 @@ cb_treeview_button_pressed (GtkTreeView * treeview, GdkEventButton * event, Xfbu
       gtk_widget_set_sensitive (menuitem_rename_artist, FALSE);
       gtk_widget_set_sensitive (menuitem_rename_title, FALSE);
     }
+#endif /* CDTEXT */
 
     gtk_menu_popup (GTK_MENU (menu_popup), NULL, NULL, NULL, NULL, event->button, gtk_get_current_event_time ());
     return TRUE;
@@ -688,6 +717,7 @@ file_exists_on_same_level (GtkTreeModel * model, GtkTreePath * path, gboolean sk
       }
 
       gtk_tree_model_get (model, &current_iter, AUDIO_COMPOSITION_COLUMN_CONTENT, &current_filename, -1);
+
       if (strcmp (current_filename, filename) == 0) {
         g_free (current_filename);
         gtk_tree_path_free (current_path);
@@ -703,7 +733,7 @@ file_exists_on_same_level (GtkTreeModel * model, GtkTreePath * path, gboolean sk
   return FALSE;
 }
 
-#if 0 // Needs libburn cd-text suppport
+#if 0 /* CDTEXT */
 static void
 cb_cell_artist_edited (GtkCellRenderer * renderer, gchar * path, gchar * newtext, XfburnAudioComposition * dc)
 {
@@ -795,6 +825,7 @@ cb_adding_done (XfburnAddingProgress *progress, XfburnAudioComposition *dc)
   xfburn_default_cursor (priv->content);
 }
 
+#if 0 /* CDTEXT */
 static void
 action_rename_selection_artist (GtkAction * action, XfburnAudioComposition * dc)
 {
@@ -844,13 +875,14 @@ action_rename_selection_title (GtkAction * action, XfburnAudioComposition * dc)
   gtk_tree_path_free (path);
   g_list_free (list);
 }
+#endif /* CDTEXT */
 
 static void
 remove_row_reference (GtkTreeRowReference *reference, XfburnAudioCompositionPrivate *priv)
 {
   GtkTreePath *path = NULL;
   GtkTreeModel *model;
-  
+
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->content));
   
   path = gtk_tree_row_reference_get_path (reference);
@@ -884,6 +916,7 @@ remove_row_reference (GtkTreeRowReference *reference, XfburnAudioCompositionPriv
       */
       
       gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
+      priv->n_tracks--;
     }
     
     gtk_tree_path_free (path);
@@ -935,7 +968,38 @@ action_add_selected_files (GtkAction *action, XfburnAudioComposition *dc)
   gchar *selected_files = NULL;
   
   xfburn_busy_cursor (priv->content);
-  selected_files = xfburn_file_browser_get_selection (browser);
+  if (xfburn_settings_get_boolean("show-filebrowser", FALSE)) {
+    selected_files = xfburn_file_browser_get_selection (browser);
+  } else {
+    GtkWidget * dialog;
+
+    dialog = gtk_file_chooser_dialog_new (_("File(s) to add to composition"),
+                                          GTK_WINDOW(xfburn_main_window_get_instance()),
+                                          GTK_FILE_CHOOSER_ACTION_OPEN,
+                                          _("Add"),
+                                          GTK_RESPONSE_ACCEPT,
+                                          NULL);
+    gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER(dialog), TRUE);
+
+    if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
+        GSList *list = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (dialog));
+        GString  * str = g_string_new(NULL);
+        GSList * curr;
+
+        for (curr = list; curr!=NULL; curr = curr->next) {
+            g_string_append(str, curr->data);
+            g_string_append_c(str, '\n');;
+        }
+
+        g_slist_free_full (list, g_free);
+        selected_files = str->str;
+        g_string_free (str, FALSE);
+        DBG("selected  files: %s ", selected_files);
+
+      //selected_files = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+    }
+    gtk_widget_destroy (dialog);
+  }
   
   if (selected_files) {
     GtkTreeSelection *selection;
@@ -954,7 +1018,7 @@ action_add_selected_files (GtkAction *action, XfburnAudioComposition *dc)
     params->model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->content));
     
     if (selected_paths) {
-      priv->path_where_insert = (GtkTreePath *) (selected_paths->data);
+      priv->path_where_insert = gtk_tree_path_copy ((GtkTreePath *) (selected_paths->data));
 
       gtk_tree_model_get_iter (params->model, &params->iter_where_insert, priv->path_where_insert);
       gtk_tree_model_get (params->model, &params->iter_where_insert, AUDIO_COMPOSITION_COLUMN_TYPE, &params->type, -1);
@@ -963,7 +1027,7 @@ action_add_selected_files (GtkAction *action, XfburnAudioComposition *dc)
     priv->selected_files = selected_files;
 
     priv->thread_params = params;
-    g_thread_create ((GThreadFunc) thread_add_files_action, params, FALSE, NULL);
+    g_thread_new ("audio_add_selected", (GThreadFunc) thread_add_files_action, params);
     
     g_list_foreach (selected_paths, (GFunc) gtk_tree_path_free, NULL);
     g_list_free (selected_paths);
@@ -981,11 +1045,20 @@ action_clear (GtkAction * action, XfburnAudioComposition * dc)
   gtk_tree_store_clear (GTK_TREE_STORE (model));
   
   xfburn_disc_usage_set_size (XFBURN_DISC_USAGE (priv->disc_usage), 0);
+  priv->n_tracks = 0;
+}
+
+static void
+action_info (GtkAction * action, XfburnAudioComposition * dc)
+{
+  XfburnAudioCompositionPrivate *priv = XFBURN_AUDIO_COMPOSITION_GET_PRIVATE (dc);
+  
+  xfce_dialog_show_info(NULL, NULL, "%s", xfburn_transcoder_get_description (priv->trans));
 }
 
 static void
 cb_content_drag_data_get (GtkWidget * widget, GdkDragContext * dc,
-                          GtkSelectionData * data, guint info, guint time, XfburnAudioComposition * content)
+                          GtkSelectionData * data, guint info, guint timestamp, XfburnAudioComposition * content)
 {
   if (info == AUDIO_COMPOSITION_DND_TARGET_INSIDE) {
     GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
@@ -1021,6 +1094,7 @@ static void
 set_modified (XfburnAudioCompositionPrivate *priv)
 {
   if (!(priv->modified)) {
+    /*
     XfburnMainWindow *mainwin;
     GtkUIManager *ui_manager;
     GtkActionGroup *action_group;
@@ -1030,7 +1104,6 @@ set_modified (XfburnAudioCompositionPrivate *priv)
   
     action_group = (GtkActionGroup *) gtk_ui_manager_get_action_groups (ui_manager)->data;
     
-    /*
     action = gtk_action_group_get_action (action_group, "save-composition");
     gtk_action_set_sensitive (GTK_ACTION (action), TRUE);
   */
@@ -1048,14 +1121,14 @@ notify_not_adding (XfburnAudioComposition * dc, GError *error)
   g_assert (error != NULL);
 
   if (error->domain != XFBURN_ERROR) {
-    xfce_warn (error->message);
+    xfce_dialog_show_warning(NULL, NULL, "%s", error->message);
     return;
   }
 
   if (g_hash_table_lookup (priv->warned_about, GINT_TO_POINTER (error->code)) == NULL) {
     g_hash_table_insert (priv->warned_about, GINT_TO_POINTER (error->code), did_warn);
 
-    xfce_warn (error->message);
+    xfce_dialog_show_warning(NULL, NULL, "%s", error->message);
   }
 }
 
@@ -1100,7 +1173,7 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnAudioComposition * d
     
     gdk_threads_enter ();
     if (file_exists_on_same_level (model, tree_path, FALSE, name)) {
-      xfce_err (_("A file with the same name is already present in the composition"));
+      xfce_dialog_show_error (NULL, NULL, _("A file with the same name is already present in the composition."));
 
       gtk_tree_path_free (tree_path);
       gdk_threads_leave ();
@@ -1178,28 +1251,34 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnAudioComposition * d
         return FALSE;
       }
 
+      if (priv->n_tracks == 99) {
+        XfburnError err_code = XFBURN_ERROR_TOO_MANY_AUDIO_TRACKS;
+
+        if (g_hash_table_lookup (priv->warned_about, GINT_TO_POINTER (err_code)) == NULL) {
+          g_hash_table_insert (priv->warned_about, GINT_TO_POINTER (err_code), did_warn);
+          gdk_threads_enter ();
+          xfce_dialog_show_error (NULL, NULL, _("You can only have a maximum of 99 tracks."));
+          gdk_threads_leave ();
+        }
+
+        return FALSE;
+      }
+
       gdk_threads_enter ();
       if (insertion != NULL) {
-        if (position == GTK_TREE_VIEW_DROP_AFTER)
+        if (position == GTK_TREE_VIEW_DROP_AFTER || position == GTK_TREE_VIEW_DROP_INTO_OR_AFTER)
           gtk_tree_store_insert_after (GTK_TREE_STORE (model), iter, NULL, insertion);
-        else if (position == GTK_TREE_VIEW_DROP_BEFORE)
+        else if (position == GTK_TREE_VIEW_DROP_BEFORE || position == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)
           gtk_tree_store_insert_before (GTK_TREE_STORE (model), iter, NULL, insertion);
         else
-          g_error ("I wasn't expecting this position!");
+          g_error ("Invalid position to drop item in!");
       } else
         gtk_tree_store_append (GTK_TREE_STORE (model), iter, NULL);
       gdk_threads_leave ();
 
       //DBG ("length = %d", atrack->length);
       secs = atrack->length;
-      humanlength = g_strdup_printf ("%2d:%2d", secs / 60, secs % 60);
-
-      if (priv->n_tracks == 99) {
-        gdk_threads_enter ();
-        xfce_err (_("You can only have a maximum of 99 tracks!"));
-        gdk_threads_leave ();
-        return FALSE;
-      }
+      humanlength = g_strdup_printf ("%2d:%02d", secs / 60, secs % 60);
 
       /* pos does not yet get recorded into atrack here, because it might
        * change still and is easier updated inside the model for now */
@@ -1218,8 +1297,13 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnAudioComposition * d
       gdk_threads_leave ();
 
       g_free (humanlength);
+      /* the tree store makes a copy of the boxed type, so we can free the original */
+      g_boxed_free (XFBURN_TYPE_AUDIO_TRACK, atrack);
 
+      gdk_threads_enter ();
       xfburn_disc_usage_add_size (XFBURN_DISC_USAGE (priv->disc_usage), secs);
+      gdk_threads_leave ();
+
       ret = TRUE;
     }
     //g_free (humansize);
@@ -1239,24 +1323,26 @@ thread_add_files_cli (ThreadAddFilesCLIParams *params)
   GtkTreeIter iter;
 
   GtkTreeModel *model;
-  int i;
-  gchar *full_path = NULL;
+  GSList * list_iter;
 
   gdk_threads_enter ();
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->content));
   gdk_threads_leave ();
 
-  for (i=0; i<params->filec; i++) {
-    full_path = g_build_filename (params->filenames[i], NULL);
+  for (list_iter = params->filelist; list_iter != NULL; list_iter = g_slist_next (list_iter)) {
+    gchar * full_path = (gchar *) list_iter->data;
+
     g_message ("Adding %s to the audio composition... (might take a while)", full_path);
     thread_add_file_to_list (params->dc, model, full_path, &iter, NULL, GTK_TREE_VIEW_DROP_AFTER);  
+
     g_free (full_path);
   }
+  g_slist_free (params->filelist);
   xfburn_adding_progress_done (XFBURN_ADDING_PROGRESS (priv->progress));
 }
 
 static gboolean
-show_add_home_question_dialog ()
+show_add_home_question_dialog (void)
 {
   gboolean ok = TRUE;
 
@@ -1282,52 +1368,57 @@ thread_add_files_action (ThreadAddFilesActionParams *params)
   GtkTreeModel *model = params->model;
   GtkTreeIter iter_where_insert = params->iter_where_insert;
   //GtkTreePath *path_where_insert = priv->path_where_insert;
-  const gchar * file = NULL;
+  gchar ** files = NULL;
+  int i;
 
 
-  file = strtok (priv->selected_files, "\n");
-  while (file) {
-    GtkTreeIter iter;
-    gchar *full_path = NULL;
-    
-    if (g_str_has_prefix (file, "file://"))
-      full_path = g_build_filename (&file[7], NULL);
-    else if (g_str_has_prefix (file, "file:"))
-      full_path = g_build_filename (&file[5], NULL);
-    else
-      full_path = g_build_filename (file, NULL);
+  files = g_strsplit (priv->selected_files, "\n", -1);
 
-    if (full_path[strlen (full_path) - 1] == '\r')
-      full_path[strlen (full_path) - 1] = '\0';
-
-    if (strcmp (full_path, g_getenv ("HOME")) == 0) {
-      if (!show_add_home_question_dialog ()) {
-        g_free (full_path);
-        break;
-      }
-    }
-
-    /* add files to the disc content */
-    if (params->type == AUDIO_COMPOSITION_TYPE_RAW) {
-      GtkTreeIter parent;
-      gboolean has_parent;
-
-      gdk_threads_enter ();
-      has_parent = gtk_tree_model_iter_parent (model, &parent, &iter_where_insert);
-      gdk_threads_leave ();
+  if (files)
+    for (i=0; files[i] != NULL && files[i][0] != '\0'; i++) {
+      GtkTreeIter iter;
+      gchar *full_path = NULL;
       
-      if (has_parent)
-        thread_add_file_to_list (dc, model, full_path, &iter, &parent, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);  
-      else 
-        thread_add_file_to_list (dc, model, full_path, &iter, NULL, GTK_TREE_VIEW_DROP_AFTER);  
-    } else {
-      thread_add_file_to_list (dc, model, full_path, &iter, NULL, GTK_TREE_VIEW_DROP_AFTER);  
-    }
-    
-    g_free (full_path);
+      if (g_str_has_prefix (files[i], "file://"))
+        full_path = g_build_filename (&files[i][7], NULL);
+      else if (g_str_has_prefix (files[i], "file:"))
+        full_path = g_build_filename (&files[i][5], NULL);
+      else
+        full_path = g_build_filename (files[i], NULL);
 
-    file = strtok (NULL, "\n");
-  }
+      if (full_path[strlen (full_path) - 1] == '\r')
+        full_path[strlen (full_path) - 1] = '\0';
+
+      if (strcmp (full_path, g_getenv ("HOME")) == 0) {
+        if (!show_add_home_question_dialog ()) {
+          g_free (full_path);
+          break;
+        }
+      }
+
+      /* add files to the disc content */
+      if (params->type == AUDIO_COMPOSITION_TYPE_RAW) {
+        GtkTreeIter parent;
+        gboolean has_parent;
+
+        gdk_threads_enter ();
+        has_parent = gtk_tree_model_iter_parent (model, &parent, &iter_where_insert);
+        gdk_threads_leave ();
+        
+        if (has_parent)
+          thread_add_file_to_list (dc, model, full_path, &iter, &parent, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);  
+        else 
+          thread_add_file_to_list (dc, model, full_path, &iter, NULL, GTK_TREE_VIEW_DROP_AFTER);  
+      } else {
+        thread_add_file_to_list (dc, model, full_path, &iter, NULL, GTK_TREE_VIEW_DROP_AFTER);  
+      }
+      
+      g_free (full_path);
+
+    }
+  /* end if files */
+
+  g_strfreev (files);
   xfburn_adding_progress_done (XFBURN_ADDING_PROGRESS (priv->progress));
 }
 
@@ -1416,7 +1507,7 @@ copy_entry_to (XfburnAudioComposition *dc, GtkTreeIter *src, GtkTreeIter *dest,
     
       /*
       if (file_exists_on_same_level (model, path_level, FALSE, name)) {
-        xfce_warn (_("A file named \"%s\" already exists in this directory, the file hasn't been added"), name);
+        xfce_dialog_warning(NULL, _("A file named \"%s\" already exists in this directory, the file hasn't been added."), name);
         goto cleanup;
       }
       */
@@ -1634,43 +1725,56 @@ cb_content_drag_data_rcv (GtkWidget * widget, GdkDragContext * dc, guint x, guin
     xfburn_default_cursor (priv->content);
   }
   /* drag from the file selector */
-  else if (sd->target == gdk_atom_intern ("text/plain", FALSE)) {
+  else if (sd->target == gdk_atom_intern ("text/plain;charset=utf-8", FALSE)) {
     ThreadAddFilesDragParams *params;
-    const gchar *file = NULL;
+    gchar **files = NULL;
     gboolean ret = FALSE;
+    gchar *full_paths;
+    int i;
 
     xfburn_adding_progress_show (XFBURN_ADDING_PROGRESS (priv->progress));
     
-    file = strtok ((gchar *) sd->data, "\n");
-    while (file) {
-      gchar *full_path;
+    full_paths = (gchar *) gtk_selection_data_get_text (sd);
 
-      if (g_str_has_prefix (file, "file://"))
-        full_path = g_build_filename (&file[7], NULL);
-      else if (g_str_has_prefix ((gchar *) sd->data, "file:"))
-        full_path = g_build_filename (&file[5], NULL);
-      else
-        full_path = g_build_filename (file, NULL);
+    files = g_strsplit ((gchar *) full_paths, "\n", -1);
 
-      if (full_path[strlen (full_path) - 1] == '\r')
-        full_path[strlen (full_path) - 1] = '\0';
+    if (files)
+      for (i=0; files[i] != NULL && files[i][0] != '\0'; i++) {
+        gchar *full_path;
 
-      /* remember path to add it later in another thread */
-      priv->full_paths_to_add = g_list_append (priv->full_paths_to_add, full_path);
-      ret = TRUE;
-      /*
-      if (xfburn_transcoder_is_audio_file (priv->trans, full_path, NULL)) {
+        if (g_str_has_prefix (files[i], "file://"))
+          full_path = g_build_filename (&files[i][7], NULL);
+        else if (g_str_has_prefix (files[i], "file:"))
+          full_path = g_build_filename (&files[i][5], NULL);
+        else
+          full_path = g_build_filename (files[i], NULL);
+
+        if (full_path[strlen (full_path) - 1] == '\r')
+          full_path[strlen (full_path) - 1] = '\0';
+
+        DBG ("Adding path '%s'", full_path);
+
+        /* remember path to add it later in another thread */
         priv->full_paths_to_add = g_list_append (priv->full_paths_to_add, full_path);
         ret = TRUE;
-      } else {
-        notify_not_adding (composition, NOT_ADDING_EXT, full_path);
+        /*
+        if (xfburn_transcoder_is_audio_file (priv->trans, full_path, NULL)) {
+          priv->full_paths_to_add = g_list_append (priv->full_paths_to_add, full_path);
+          ret = TRUE;
+        } else {
+          notify_not_adding (composition, NOT_ADDING_EXT, full_path);
+        }
+        */
+
       }
-      */
-
-      file = strtok (NULL, "\n");
-    }
+    /* end if files */
+    g_strfreev (files);
+    g_free (full_paths);
 
+    /* paths actually get sent to us in reverse order,
+     * so no reverse is necessary to add them in the same order as selected.
     priv->full_paths_to_add = g_list_reverse (priv->full_paths_to_add);
+     */
     priv->path_where_insert = path_where_insert;
 
     if (ret) {
@@ -1684,7 +1788,7 @@ cb_content_drag_data_rcv (GtkWidget * widget, GdkDragContext * dc, guint x, guin
       gtk_tree_store_append (GTK_TREE_STORE (model), &params->iter_dummy, NULL);
 
       priv->thread_params = params;
-      g_thread_create ((GThreadFunc) thread_add_files_drag, params, FALSE, NULL);
+      g_thread_new ("add_files_drag", (GThreadFunc) thread_add_files_drag, params);
 
       gtk_drag_finish (dc, TRUE, FALSE, t);
     } else {
@@ -1693,38 +1797,46 @@ cb_content_drag_data_rcv (GtkWidget * widget, GdkDragContext * dc, guint x, guin
     }
   } 
   else if (sd->target == gdk_atom_intern ("text/uri-list", FALSE)) {
-#ifdef HAVE_THUNAR_VFS
     GList *vfs_paths = NULL;
     GList *vfs_path;
-    GError *error = NULL;
+    GList *lp;
     gchar *full_path;
+    gchar **uris;
+    gsize   n;
     gboolean ret = FALSE;
 
-    vfs_paths = thunar_vfs_path_list_from_string ((gchar *) sd->data, &error);
+    uris = g_uri_list_extract_uris ((gchar *) sd->data);
+
+    for (n = 0; uris != NULL && uris[n] != NULL; ++n)
+      vfs_paths = g_list_append (vfs_paths, g_file_new_for_uri (uris[n]));
+
+    g_strfreev (uris);
 
     if (G_LIKELY (vfs_paths != NULL)) {
       ThreadAddFilesDragParams *params;
       priv->full_paths_to_add = NULL;
       for (vfs_path = vfs_paths; vfs_path != NULL; vfs_path = g_list_next (vfs_path)) {
-        ThunarVfsPath *path = THUNAR_VFS_PATH (vfs_path->data);
-        if (thunar_vfs_path_get_scheme (path) != THUNAR_VFS_PATH_SCHEME_FILE)
+	GFile *path = vfs_path->data;
+	if (path == NULL)
           continue;
-        full_path = thunar_vfs_path_dup_string (path);
+	/* unable to handle non-local files */
+	if (G_UNLIKELY (!g_file_has_uri_scheme (path, "file"))) {
+            g_object_unref (path);
+	    continue;
+        }
+        full_path = g_file_get_path (path);
+        /* if there is no local path, use the URI (which always works) */
+        if (full_path == NULL)
+            full_path = g_file_get_uri (path);
+        /* release the location */
         g_debug ("adding uri path: %s", full_path);
-
         priv->full_paths_to_add = g_list_prepend (priv->full_paths_to_add, full_path);
         ret = TRUE;
-        /*
-        if (xfburn_transcoder_is_audio_file (priv->trans, full_path, NULL)) {
-          priv->full_paths_to_add = g_list_prepend (priv->full_paths_to_add, full_path);
-          ret = TRUE;
-        } else {
-          g_error_free (error);
-          notify_not_adding (composition, NOT_ADDING_EXT, full_path);
-        }
-        */
       }
-      thunar_vfs_path_list_free (vfs_paths);
+
+      for (lp = vfs_paths; lp != NULL; lp = lp->next)
+        g_object_unref (lp->data);
+      g_list_free (vfs_paths);
 
       priv->full_paths_to_add = g_list_reverse (priv->full_paths_to_add);
       priv->path_where_insert = path_where_insert;
@@ -1739,7 +1851,7 @@ cb_content_drag_data_rcv (GtkWidget * widget, GdkDragContext * dc, guint x, guin
         gtk_tree_store_append (GTK_TREE_STORE (model), &params->iter_dummy, NULL);
 
         priv->thread_params = params;
-        g_thread_create ((GThreadFunc) thread_add_files_drag, params, FALSE, NULL);
+        g_thread_new ("audio_add_uri_list", (GThreadFunc) thread_add_files_drag, params);
 
         gtk_drag_finish (dc, TRUE, FALSE, t);
       } else {
@@ -1747,18 +1859,10 @@ cb_content_drag_data_rcv (GtkWidget * widget, GdkDragContext * dc, guint x, guin
         cb_adding_done (XFBURN_ADDING_PROGRESS (priv->progress), composition);
       }
     } else {
-      if (G_UNLIKELY (error != NULL))
-        g_warning ("text/uri-list drag failed because '%s'", error->message);
-      else
-        g_warning("There were no files in the uri list!");
+      g_warning("There were no files in the uri list!");
       gtk_drag_finish (dc, FALSE, FALSE, t);
       xfburn_default_cursor (priv->content);
     }
-#else
-    g_warning ("Receiving this type of drag and drop requires thunar-vfs support, sorry!");
-    gtk_drag_finish (dc, FALSE, FALSE, t);
-    xfburn_default_cursor (priv->content);
-#endif
   } 
   else {
     g_warning ("Trying to receive an unsupported drag target, this should not happen.");
@@ -1822,6 +1926,7 @@ typedef struct
   GQueue *queue_iter;
 } LoadParserStruct;
 
+/*
 static gint
 _find_attribute (const gchar ** attribute_names, const gchar * attr)
 {
@@ -1834,16 +1939,19 @@ _find_attribute (const gchar ** attribute_names, const gchar * attr)
 
   return -1;
 }
+*/
 
 static void
 load_composition_start (GMarkupParseContext * context, const gchar * element_name,
                         const gchar ** attribute_names, const gchar ** attribute_values,
                         gpointer data, GError ** error)
 {
+      g_error ("This method needs to get fixed, and does not work right now!");
+  
+      /*
   LoadParserStruct * parserinfo = (LoadParserStruct *) data;
   XfburnAudioCompositionPrivate *priv = XFBURN_AUDIO_COMPOSITION_GET_PRIVATE (parserinfo->dc);
 
-  
   if (!(parserinfo->started) && !strcmp (element_name, "xfburn-composition"))
     parserinfo->started = TRUE;
   else if (!(parserinfo->started))
@@ -1861,11 +1969,8 @@ load_composition_start (GMarkupParseContext * context, const gchar * element_nam
       model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->content));
       parent = g_queue_peek_head (parserinfo->queue_iter);
           
-      g_error ("This method needs to get fixed, and does not work right now!");
-      /*
       add_file_to_list_with_name (attribute_values[i], parserinfo->dc, model, attribute_values[j], &iter, 
                                   parent, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
-      */
     }
   } else if (!strcmp (element_name, "directory")) {
     int i, j;
@@ -1880,6 +1985,7 @@ load_composition_start (GMarkupParseContext * context, const gchar * element_nam
       //add_directory_to_list (attribute_values[i], parserinfo->dc, model, attribute_values[j], &iter, parent);    
     }
   }
+      */
 }
 
 static void
@@ -2063,23 +2169,22 @@ xfburn_audio_composition_new (void)
 }
 
 void 
-xfburn_audio_composition_add_files (XfburnAudioComposition *dc, int filec, char **filenames)
+xfburn_audio_composition_add_files (XfburnAudioComposition *dc, GSList * filelist)
 {
   XfburnAudioCompositionPrivate *priv = XFBURN_AUDIO_COMPOSITION_GET_PRIVATE (dc);
   ThreadAddFilesCLIParams *params;
   
-  if (filec > 0) {
+  if (filelist != NULL) {
     params = g_new (ThreadAddFilesCLIParams, 1);
 
-    params->filenames = filenames;
-    params->filec = filec;
+    params->filelist = filelist;
     params->dc = dc;
 
     xfburn_adding_progress_show (XFBURN_ADDING_PROGRESS (priv->progress));
     xfburn_busy_cursor (priv->content);
 
     priv->thread_params = params;
-    g_thread_create ((GThreadFunc) thread_add_files_cli, params, FALSE, NULL);
+    g_thread_new ("audio_add_files_cli", (GThreadFunc) thread_add_files_cli, params);
   }
 }
 
