@@ -144,6 +144,7 @@ static gboolean thread_add_file_to_list_with_name (const gchar *name, XfburnData
                                                    GtkTreeIter * insertion, GtkTreeViewDropPosition position);
 static gboolean thread_add_file_to_list (XfburnDataComposition * dc, GtkTreeModel * model, const gchar * path, 
                                          GtkTreeIter * iter, GtkTreeIter * insertion, GtkTreeViewDropPosition position);
+static void concat_free (char * msg, char ** combined_msg);
 static IsoImage * generate_iso_image (XfburnDataComposition * dc);
 static gboolean show_add_home_question_dialog (void);
                                   
@@ -175,10 +176,12 @@ typedef struct
   GtkWidget *progress;
   GtkTreeStore *model;
   GtkWidget *add_filechooser;
+  gchar * last_directory;
   GtkWidget *add_window;
 
   gchar *default_vol_name;
 
+  gboolean large_files;
 } XfburnDataCompositionPrivate;
 
 /* globals */
@@ -911,6 +914,8 @@ add_cb (GtkWidget * widget, gpointer data)
     GString  * str = g_string_new(NULL);
     GSList * curr;
 
+    priv->last_directory = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER(priv->add_filechooser));
+
     for (curr = list; curr!=NULL; curr = curr->next) {
         g_string_append(str, curr->data);
         g_string_append_c(str, '\n');;
@@ -946,6 +951,8 @@ select_files (XfburnDataComposition * dc)
     priv->add_filechooser = gtk_file_chooser_widget_new (GTK_FILE_CHOOSER_ACTION_OPEN);
 
     gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER(priv->add_filechooser), TRUE);
+    if (priv->last_directory)
+      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(priv->add_filechooser), priv->last_directory);
 
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_container_add(GTK_CONTAINER(window), vbox);
@@ -1278,8 +1285,13 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnDataComposition * dc
         gdk_threads_leave ();
 
         return FALSE;
+      } else if (s.st_size > MAXIMUM_ISO_LEVEL_2_FILE_SIZE && !priv->large_files) {
+        priv->large_files = TRUE;
+        gdk_threads_enter ();
+        xfce_dialog_show_warning (NULL, NULL, _("%s is larger than what iso9660 level 2 allows. This can be a problem for old systems or software."), path);
+        gdk_threads_leave ();
       }
-	  
+
       gdk_threads_enter ();
       screen = gtk_widget_get_screen (GTK_WIDGET (dc));
       icon_theme = gtk_icon_theme_get_for_screen (screen);
@@ -1893,7 +1905,7 @@ thread_add_files_drag (ThreadAddFilesDragParams *params)
 }
 
 static void
-fill_image_with_composition (GtkTreeModel *model, IsoImage *image, IsoDir * parent, GtkTreeIter *iter)
+fill_image_with_composition (GtkTreeModel *model, IsoImage *image, IsoDir * parent, GtkTreeIter *iter, GSList **errors)
 {
   do {
       DataCompositionEntryType type;
@@ -1922,14 +1934,23 @@ fill_image_with_composition (GtkTreeModel *model, IsoImage *image, IsoDir * pare
       }
 
       if (r < 0) {
+        char * msg;
+
         if (r == ISO_NULL_POINTER)
-          g_error ("Failed adding %s as a node to the image: null pointer!", src);
-        else if (r == ISO_NODE_NAME_NOT_UNIQUE)
-          g_error ("Failed adding %s as a node to the image: node name not unique!", src);
+          g_error (_("%s: null pointer"), src);
         else if (r == ISO_OUT_OF_MEM)
-          g_error ("Failed adding %s as a node to the image: out of memory!", src);
+          g_error (_("%s: out of memory"), src);
+        else if (r == ISO_NODE_NAME_NOT_UNIQUE)
+          msg = g_strdup_printf (_("%s: node name not unique"), src);
         else
-          g_error ("Failed adding %s as a node to the image: code %X!", src, r);
+          msg = g_strdup_printf (_("%s: %s (code %X)"), src, iso_error_to_msg(r), r);
+
+        g_warning ("%s", msg);
+        *errors = g_slist_prepend (*errors, msg);
+
+        g_free (name);
+        g_free (src);
+        continue;
       }
 
       if (src != '\0') {
@@ -1967,11 +1988,18 @@ fill_image_with_composition (GtkTreeModel *model, IsoImage *image, IsoDir * pare
         dir = (IsoDir *)node;
 
 	gtk_tree_model_iter_children (model, &child, iter);
-	fill_image_with_composition (model, image, dir, &child);
+	fill_image_with_composition (model, image, dir, &child, errors);
       }
   } while (gtk_tree_model_iter_next (model, iter));
 }
 
+static
+void concat_free (char * msg, char ** combined_msg)
+{
+  *combined_msg = g_strconcat (*combined_msg, msg, "\n", NULL);
+  free (msg);
+}
+
 static IsoImage *
 generate_iso_image (XfburnDataComposition * dc)
 {
@@ -1979,6 +2007,7 @@ generate_iso_image (XfburnDataComposition * dc)
   IsoImage *image = NULL;
   GtkTreeModel *model;
   GtkTreeIter iter;
+  GSList *errors = NULL;
 
   iso_image_new (gtk_entry_get_text (GTK_ENTRY (priv->entry_volume_name)), &image);
   iso_image_set_application_id (image, "Xfburn");
@@ -1986,7 +2015,54 @@ generate_iso_image (XfburnDataComposition * dc)
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->content));
   if (gtk_tree_model_get_iter_first (model, &iter)) {
-    fill_image_with_composition (model, image, iso_image_get_root (image), &iter);
+    fill_image_with_composition (model, image, iso_image_get_root (image), &iter, &errors);
+
+    if (errors != NULL) {
+      gchar * combined_msg = "";
+      GtkWidget * dialog, * label, * textview, * scrolled;
+      GtkTextBuffer * buffer;
+      XfburnMainWindow *mainwin = xfburn_main_window_get_instance ();
+      gchar * title;
+
+      errors = g_slist_reverse (errors);
+
+      g_slist_foreach (errors, (GFunc) concat_free, &combined_msg);
+
+      title = _("Error(s) occured while adding files");
+      dialog = gtk_dialog_new_with_buttons (title,
+                                            GTK_WINDOW (mainwin),
+                                            GTK_DIALOG_DESTROY_WITH_PARENT,
+                                            GTK_STOCK_OK,
+                                            GTK_RESPONSE_OK,
+                                            NULL);
+
+      label = gtk_label_new (NULL);
+      title = g_strdup_printf ("<b>%s</b>", title);
+      gtk_label_set_markup(GTK_LABEL (label), title);
+      g_free (title);
+      gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);
+
+      textview = gtk_text_view_new ();
+      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
+      gtk_text_buffer_set_text (buffer, combined_msg, -1);
+      gtk_text_view_set_editable (GTK_TEXT_VIEW(textview), FALSE);
+
+      scrolled = gtk_scrolled_window_new (NULL, NULL);
+      gtk_container_add (GTK_CONTAINER (scrolled), textview);
+      gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+      gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), scrolled);
+
+      gtk_window_set_default_size (GTK_WINDOW (dialog), 600, 200);
+
+      gtk_widget_show (label);
+      gtk_widget_show (textview);
+      gtk_widget_show (scrolled);
+
+      gtk_dialog_run (GTK_DIALOG (dialog));
+
+      gtk_widget_destroy (dialog);
+      g_free (combined_msg);
+    }
   }
 
   return image;
