@@ -44,6 +44,7 @@ enum {
   PROP_DVDR,
   PROP_DVDPLUSR,
   PROP_DVDRAM,
+  PROP_BD,
   PROP_TAO_BLOCK_TYPES,
   PROP_SAO_BLOCK_TYPES,
   PROP_RAW_BLOCK_TYPES,
@@ -80,6 +81,7 @@ struct _XfburnDevicePrivate {
   gboolean dvdr;
   gboolean dvdplusr;
   gboolean dvdram;
+  gboolean bd;
 
   GSList *supported_speeds;
 
@@ -137,6 +139,9 @@ xfburn_device_get_property (GObject *object, guint property_id,
     case PROP_DVDRAM:
       g_value_set_boolean (value, priv->dvdram);
       break;
+    case PROP_BD:
+      g_value_set_boolean (value, priv->bd);
+      break;
     case PROP_TAO_BLOCK_TYPES:
       g_value_set_int (value, priv->tao_block_types);
       break;
@@ -197,6 +202,9 @@ xfburn_device_set_property (GObject *object, guint property_id,
     case PROP_DVDRAM:
       priv->dvdram = g_value_get_boolean (value);
       break;
+    case PROP_BD:
+      priv->bd = g_value_get_boolean (value);
+      break;
     case PROP_TAO_BLOCK_TYPES:
       priv->tao_block_types = g_value_get_int (value);
       break;
@@ -274,6 +282,9 @@ xfburn_device_class_init (XfburnDeviceClass *klass)
   g_object_class_install_property (object_class, PROP_DVDRAM, 
                                    g_param_spec_boolean ("dvdram", _("Can burn DVDRAM"),
                                                         _("Can burn DVDRAM"), FALSE, G_PARAM_READWRITE));
+  g_object_class_install_property (object_class, PROP_BD,
+                                   g_param_spec_boolean ("bd", _("Can burn Blu-ray"),
+                                                        _("Can burn Blu-ray"), FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (object_class, PROP_TAO_BLOCK_TYPES, 
                                    g_param_spec_int ("tao-block-types", _("libburn TAO block types"),
                                                      _("libburn TAO block types"), 0, G_MAXINT, 0, G_PARAM_READABLE));
@@ -413,6 +424,8 @@ xfburn_device_fillin_libburn_info (XfburnDevice *device, struct burn_drive_info
   priv->buffer_size = drive->buffer_size;
   priv->dummy_write = drive->write_simulate;
 
+  DBG ("libburn will determine BD support based on the disk in the drive");
+
   /* write modes */
   priv->tao_block_types = drive->tao_block_types;
   priv->sao_block_types = drive->sao_block_types;
@@ -511,6 +524,7 @@ xfburn_device_release (struct burn_drive_info *drive_info, gboolean eject)
 {
   int ret;
 
+  burn_drive_snooze (drive_info->drive, 0);
   burn_drive_release (drive_info->drive, eject);
 
   ret = burn_drive_info_forget (drive_info, 0);
