@@ -41,6 +41,7 @@
 
 #include <gst/gst.h>
 #include <gst/pbutils/missing-plugins.h>
+#include <gst/pbutils/pbutils.h>
 
 #include "xfburn-global.h"
 #include "xfburn-error.h"
@@ -84,9 +85,6 @@ static void finish (XfburnTranscoder *trans);
 
 static gboolean is_initialized (XfburnTranscoder *trans, GError **error);
 
-static gboolean signal_identification_done (XfburnTranscoderGst *trans, const char *dbg_res);
-static gboolean query_and_signal_duration (XfburnTranscoderGst *trans);
-
 /* gstreamer support functions */
 static gboolean bus_call (GstBus *bus, GstMessage *msg, gpointer data);
 static void on_pad_added (GstElement *element, GstPad *pad, gpointer data);
@@ -103,7 +101,6 @@ enum {
 
 typedef enum {
   XFBURN_TRANSCODER_GST_STATE_IDLE,
-  XFBURN_TRANSCODER_GST_STATE_IDENTIFYING,
   XFBURN_TRANSCODER_GST_STATE_TRANSCODE_START,
   XFBURN_TRANSCODER_GST_STATE_TRANSCODING,
 } XfburnTranscoderGstState;
@@ -116,9 +113,10 @@ typedef struct {
   GCond gst_cond;
   GMutex gst_mutex;
   gboolean gst_done;
-  gboolean is_audio;
   gint64 duration;
 
+  GstDiscoverer *discoverer;
+
   GError *error;
 
   GSList *tracks;
@@ -147,6 +145,11 @@ static guint64 total_size = 0;
 
 static const gchar *errormsg_gst_setup = "An error occurred setting gstreamer up for transcoding";
 static const gchar *errormsg_libburn_setup = "An error occurred while setting the burning backend up";
+static const gchar *errormsg_missing_plugin = "%s is missing.\n"
+                                              "\n"
+                                              "You do not have a decoder installed to handle this file.\n"
+                                              "Probably you need to look at the gst-plugins-* packages\n"
+                                              "for the necessary plugins.\n";
 
 /*********************/
 /* class declaration */
@@ -220,6 +223,8 @@ xfburn_transcoder_gst_init (XfburnTranscoderGst * obj)
    * information from gst */
   g_mutex_init (&priv->gst_mutex);
   g_mutex_lock (&priv->gst_mutex);
+
+  priv->discoverer = gst_discoverer_new(GST_SECOND, NULL);
 }
 
 static void
@@ -230,6 +235,7 @@ xfburn_transcoder_gst_finalize (GObject * object)
   gst_element_set_state (priv->pipeline, GST_STATE_NULL);
 
   gst_object_unref (GST_OBJECT (priv->pipeline));
+  gst_object_unref (GST_OBJECT (priv->discoverer));
   priv->pipeline = NULL;
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -388,82 +394,6 @@ state_to_str (GstState st)
 }
 #endif
 
-/*
- * this function expects the result to be in priv->is_audio, 
- * dbg_res is only for debugging output
- */
-static gboolean
-signal_identification_done (XfburnTranscoderGst *trans, const char *dbg_res)
-{
-  XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (trans);
-
-  int i;
-
-#if DEBUG_GST > 0
-  DBG ("Trying to lock mutex (%s)", dbg_res);
-#endif
-
-  priv->gst_done = TRUE;
-
-  /* There is no g_mutex_lock_timed, so emulate it with a loop.
-    * I have never seen this getting hung here, but one never knows! */
-  for (i=0; i<SIGNAL_SEND_ITERATIONS; i++) {
-    if (g_mutex_trylock (&priv->gst_mutex))
-      break;
-    g_usleep (SIGNAL_SEND_TIMEOUT_MICROS / SIGNAL_SEND_ITERATIONS);
-    g_thread_yield ();
-    if (i==9) {
-      g_warning ("Noone was there to listen to the result of the identification!");
-      /* FIXME: recreate pipeline here? This state is not the fault of gst */
-      if (gst_element_set_state (priv->pipeline, GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE) {
-        DBG ("Oops, could not reset pipeline to null");
-      }
-      return FALSE;
-    }
-  }
-
-  g_cond_signal (&priv->gst_cond);
-  g_mutex_unlock (&priv->gst_mutex);
-
-#if DEBUG_GST > 0
- #if DEBUG > 0
-  DBG ("Releasing mutex (%s)", dbg_res);
- #else
-  g_message ("Signaled identification done: %s", dbg_res);
- #endif
-#endif
-
-  return TRUE;
-}
-
-
-static gboolean
-query_and_signal_duration (XfburnTranscoderGst *trans)
-{
-  XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (trans);
-  GstFormat fmt;
-  //guint secs;
-
-  fmt = GST_FORMAT_TIME;
-  if (!gst_element_query_duration (priv->pipeline, fmt, &priv->duration)) {
-    return FALSE;
-  }
-
-  //secs = priv->duration / 1000000000;
-  if (gst_element_set_state (priv->pipeline, GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE) {
-    DBG ("Failed to set state!");
-    recreate_pipeline (trans);
-  }
-
-  priv->is_audio = TRUE;
-  priv->state = XFBURN_TRANSCODER_GST_STATE_IDLE;
-
-  signal_identification_done (trans, "is audio");
-
-  return TRUE;
-}
-
-
 static gboolean
 bus_call (GstBus *bus, GstMessage *msg, gpointer data)
 {
@@ -519,16 +449,6 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
           recreate_pipeline (trans);
           break;
 
-        case XFBURN_TRANSCODER_GST_STATE_IDENTIFYING:
-          recreate_pipeline (trans);
-
-          g_message ("gstreamer abort: %s\n", error->message);
-          priv->is_audio = FALSE;
-          priv->error = error;
-
-          signal_identification_done (trans, "error");
-          break;
-
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODE_START:
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODING:
           g_error ("Gstreamer error while transcoding: %s", error->message);
@@ -543,7 +463,7 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
         DBG ("Failed to reset GST state.");
         recreate_pipeline (trans);
       }
-      signal_identification_done (trans, "not audio content");
+      DBG("GST recognized an application, ignoring.");
 
       break;
     }
@@ -566,18 +486,6 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODING:
           break;
 
-        case XFBURN_TRANSCODER_GST_STATE_IDENTIFYING:
-          if (state != GST_STATE_PAUSED)
-            break;
-          if (!query_and_signal_duration (trans)) {
-            /* this is expected to fail a couple of times,
-             * so no output unless we're in debug mode */
-#if DEBUG_GST > 1
-            DBG ("could not query stream duration (expected failure)");
-#endif
-          }
-          break;
-
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODE_START:
           if (state != GST_STATE_PLAYING)
             break;
@@ -611,34 +519,20 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODE_START:
           break;
 
-        case XFBURN_TRANSCODER_GST_STATE_IDENTIFYING:
-          if (!query_and_signal_duration (trans)) {
-            /* this is expected to work, because we got the duration message on the bus */
-#if DEBUG_GST > 0
-            g_warning ("Could not query stream length!");
-#endif
-            return TRUE;
-          }
-          break;
       } /* switch of priv->state */
 
       break;
     }
 
     case GST_MESSAGE_ELEMENT: {
+      /* This should not happen now that the discoverer API is used to identify songs.
+       * Shouldn't hurt to keep around though. */
       if (gst_is_missing_plugin_message (msg)) {
           recreate_pipeline (trans);
 
-          priv->is_audio = FALSE;
           g_set_error (&(priv->error), XFBURN_ERROR, XFBURN_ERROR_MISSING_PLUGIN,
-                       _("%s is missing.\n"
-                         "\n"
-                         "You do not have a decoder installed to handle this file.\n"
-                         "Probably you need to look at the gst-plugins-* packages\n"
-                         "for the necessary plugins.\n"),
+                       _(errormsg_missing_plugin),
                         gst_missing_plugin_message_get_description (msg)); 
-
-          signal_identification_done (trans, "missing-plugin");
       }
     }
     default:
@@ -689,7 +583,6 @@ on_pad_added (GstElement *element, GstPad *pad, gpointer data)
     gst_caps_unref (caps);
     gst_object_unref (audiopad);
 
-    priv->is_audio = FALSE;
     g_set_error (&(priv->error), XFBURN_ERROR, XFBURN_ERROR_GST_NO_AUDIO,
 		    "%s",
                  _(error_msg));
@@ -758,6 +651,26 @@ cb_handoff (GstElement *element, GstBuffer *buffer, gpointer data)
 
 #endif
 
+static gchar *
+get_discoverer_required_plugins_message (GstDiscovererInfo *info)
+{
+  GString *str;
+  gchar **plugins;
+  gchar *plugins_str;
+
+  plugins = (gchar **) gst_discoverer_info_get_missing_elements_installer_details (info);
+
+  if (g_strv_length(plugins) == 0) {
+    str = g_string_new ("No information available on which plugin is required.");
+  } else {
+    str = g_string_new("Required plugins: ");
+    plugins_str = g_strjoinv (", ", plugins);
+    g_string_append (str, plugins_str);
+    g_free (plugins_str);
+  }
+
+  return g_string_free (str, FALSE);
+}
 
 static gboolean
 get_audio_track (XfburnTranscoder *trans, XfburnAudioTrack *atrack, GError **error)
@@ -766,47 +679,62 @@ get_audio_track (XfburnTranscoder *trans, XfburnAudioTrack *atrack, GError **err
   XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (tgst);
 
   XfburnAudioTrackGst *gtrack;
-  gint64 end_time;
   off_t size;
+  GstDiscovererInfo *info;
+  GstDiscovererResult result;
+  gchar *uri;
 
-  priv->is_audio = FALSE;
 #if DEBUG_GST > 0
   DBG ("Querying GST about %s", atrack->inputfile);
 #endif
 
-  priv->state = XFBURN_TRANSCODER_GST_STATE_IDENTIFYING;
-  g_object_set (G_OBJECT (priv->source), "location", atrack->inputfile, NULL);
-  if (gst_element_set_state (priv->pipeline, GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE) {
-#if DEBUG_GST > 0
-    g_message ("Supposedly failed to change gstreamer state, ignoring it as usually it does it anyways.");
-#endif
-  }
+  uri = g_strdup_printf("file://%s", atrack->inputfile);
+  info = gst_discoverer_discover_uri(priv->discoverer, uri, error);
+  g_free(uri);
 
-  end_time = g_get_monotonic_time () + SIGNAL_WAIT_TIMEOUT_MS * G_TIME_SPAN_MILLISECOND;
-#if DEBUG_GST > 0
-  DBG ("Now waiting for identification result");
-#endif
-  while (!priv->gst_done)
-    if (!g_cond_wait_until (&priv->gst_cond, &priv->gst_mutex, end_time)) {
-      DBG ("gst identification timed out");
-      recreate_pipeline (tgst);
-      g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_TIMEOUT,
-                   _("Gstreamer did not like this file (detection timed out)"));
-      return FALSE;
+  if (info == NULL) {
+    if (error && *error == NULL) {
+        DBG ("no info and no error");
+        g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_DISCOVERER,
+                    _("An error occurred while identifying '%s' with gstreamer"), atrack->inputfile);
     }
-  priv->gst_done = FALSE;
-#if DEBUG_GST > 0
-  DBG ("Got an identification result ");
-#endif
-
-  if (!priv->is_audio) {
-    g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_NOT_AUDIO_FORMAT,
-                 _("%s\n\nis not an audio file:\n\n%s"), atrack->inputfile, priv->error->message);
-    g_error_free (priv->error);
-    priv->error = NULL;
     return FALSE;
   }
 
+  result = gst_discoverer_info_get_result(info);
+  switch (result) {
+    case GST_DISCOVERER_MISSING_PLUGINS:
+      g_clear_error (error);
+      /*
+      g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_DISCOVERER,
+                  _("%s: %s"), atrack->inputfile, get_discoverer_required_plugins_message (info));
+      */
+      // the message is pretty useless, so print it only on the console, and create our own instead.
+      DBG ("%s", get_discoverer_required_plugins_message (info));
+      g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_MISSING_PLUGIN,
+                   _(errormsg_missing_plugin),
+                   _("A plugin"));
+
+      gst_discoverer_info_unref(info);
+      return FALSE;
+    case GST_DISCOVERER_OK:
+      break;
+    default:
+      gst_discoverer_info_unref(info);
+      DBG ("gst discoverer said %d", result);
+      /* TODO: improve error messages */
+      //recreate_pipeline (tgst);
+      if (error && *error == NULL)
+        g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_DISCOVERER,
+                     _("An error occurred while identifying '%s' with gstreamer"), atrack->inputfile);
+      else
+        g_prefix_error (error, "%s: ", atrack->inputfile);
+      return FALSE;
+  }
+
+  priv->gst_done = FALSE;
+
+  priv->duration = gst_discoverer_info_get_duration(info);
   atrack->length = priv->duration / 1000000000;
 
   size = (off_t) floorf (priv->duration * (PCM_BYTES_PER_SECS / (float) 1000000000));
@@ -821,6 +749,8 @@ get_audio_track (XfburnTranscoder *trans, XfburnAudioTrack *atrack, GError **err
 
   gtrack->size = size;
 
+  gst_discoverer_info_unref(info);
+
   return TRUE;
 }
 
